@@ -1,7 +1,7 @@
 package App::wordlist;
 
-our $DATE = '2016-02-29'; # DATE
-our $VERSION = '0.21'; # VERSION
+our $DATE = '2016-04-15'; # DATE
+our $VERSION = '0.22'; # VERSION
 
 use 5.010001;
 use strict;
@@ -18,7 +18,34 @@ sub _list_installed {
             list_pod      => 0,
             recurse       => 1,
         });
-    [map {s/\AWordList:://; $_} sort keys %$mods];
+    my @res;
+    for my $wl0 (sort keys %$mods) {
+        $wl0 =~ s/\AWordList:://;
+        my $wl = $wl0;
+
+        my $type;
+        if ($wl =~ s/^Char:://) {
+            $type = 'Char';
+        } elsif ($wl =~ s/^Phrase:://) {
+            $type = 'Phrase';
+        } elsif ($wl =~ s/^Test:://) {
+            $type = 'Test';
+        } else {
+            $type = 'Word';
+        }
+
+        my $lang = '';
+        if ($wl =~ /^(\w\w)::/) {
+            $lang = $1;
+        }
+
+        push @res, {
+            name => $wl0,
+            lang => $lang,
+            type => $type,
+        };
+     }
+    \@res;
 }
 
 $SPEC{wordlist} = {
@@ -43,7 +70,9 @@ $SPEC{wordlist} = {
         max_len => {
             schema  => 'int*',
         },
-        wordlist => {
+        wordlists => {
+            'x.name.is_plural' => 1,
+            'x.name.singular' => 'wordlist',
             schema => ['array*' => of => 'str*'],
             summary => 'Select one or more wordlist modules',
             cmdline_aliases => {w=>{}},
@@ -53,7 +82,7 @@ $SPEC{wordlist} = {
                 my %args = @_;
                 Complete::Util::complete_array_elem(
                     word  => $args{word},
-                    array => _list_installed(),
+                    array => [map {$_->{name}} @{ _list_installed() }],
                     ci    => 1,
                 );
             },
@@ -83,12 +112,49 @@ $SPEC{wordlist} = {
         },
         lcpan => {
             schema => 'bool',
-            summary => 'Use local CPAN mirror first when available',
+            summary => 'Use local CPAN mirror first when available (for -L)',
         },
         detail => {
             summary => 'Display more information when listing modules',
             schema  => 'bool',
         },
+        type => {
+            summary => 'Only include wordlists of certain type',
+            description => <<'_',
+
+By convention, type information is encoded in the wordlist's name. `Char` means
+to only include wordlists with names matching `Char::*`. `Phrase` means to only
+include wordlists with names matching `Phrase::*`. `Word` means to only include
+wordlists that are not of type `Char` and `Phrase`. `Test` means to only include
+wordlists with names matching `Test::*`.
+
+_
+            schema => ['str*', in=>['Char', 'Phrase', 'Word', 'Test']],
+            description => <<'_',
+
+By convention, language information is encoded in the wordlist's name. For
+example, English wordlists have names matching `EN::*` or `Word::EN::*` or
+`Char::EN::*` or `Phrase::EN::*`.
+
+_
+            cmdline_aliases => {t=>{}},
+        },
+        lang => {
+            summary => 'Only include wordlists of certain language',
+            schema => ['str*', match => '\A\w\w\z'],
+            completion => sub {
+                my %args = @_;
+                my @langs;
+                for (@{ _list_installed() }) {
+                    next unless length $_->{lang};
+                    push @langs, $_->{lang}
+                        unless grep {$_ eq $_->{name}} @langs;
+                }
+                require Complete::Util;
+                Complete::Util::complete_array_elem(
+                    word => $args{word}, array => \@langs);
+            },
+        },
     },
     examples => [
         {
@@ -115,6 +181,18 @@ $SPEC{wordlist} = {
             test => 0,
             'x.doc.show_result' => 0,
         },
+        {
+            argv => [qw/-t Phrase foo/],
+            summary => 'Select phrase wordlists',
+            test => 0,
+            'x.doc.show_result' => 0,
+        },
+        {
+            argv => [qw/--lang FR foo/],
+            summary => 'Select French wordlists',
+            test => 0,
+            'x.doc.show_result' => 0,
+        },
         {
             argv => [qw|/fof[aeiou]/|],
             summary => 'Filter by regex',
@@ -159,9 +237,20 @@ sub wordlist {
         }
 
         my @res;
-        my $wordlists = $args{wordlist};
-        if (!$wordlists || !@$wordlists) {
-            $wordlists = $list_installed;
+        my $wordlists;
+        if ($args{wordlists}) {
+            $wordlists = $args{wordlists};
+        } else {
+            $wordlists = [];
+            for my $rec (@$list_installed) {
+                if ($args{type}) {
+                    next unless $rec->{type} eq $args{type};
+                }
+                if ($args{lang}) {
+                    next unless $rec->{lang} eq uc($args{lang});
+                }
+                push @$wordlists, $rec->{name};
+            }
         }
         for my $wl (@$wordlists) {
             my $mod = "WordList::$wl";
@@ -210,11 +299,9 @@ sub wordlist {
         my @res;
         for (@$list_installed) {
             if ($args{detail}) {
-                push @res, {
-                    name   => $_,
-                };
-            } else {
                 push @res, $_;
+            } else {
+                push @res, $_->{name};
             }
         }
         [200, "OK", \@res,
@@ -253,10 +340,11 @@ sub wordlist {
                 my @res;
                 while (my $row = $rs->next) {
                     my $mod = $row->module->[0]{name};
-                    next unless grep {$mod eq $_} @res;
+                    say "D: mod=$mod" if $ENV{DEBUG};
                     $mod =~ s/\AWordList:://;
-                    push @res, $mod;
+                    push @res, $mod unless grep {$mod eq $_} @res;
                 }
+                warn "Empty result from MetaCPAN\n" unless @res;
                 return [200, "OK", [sort @res]];
             }
         }
@@ -284,7 +372,7 @@ App::wordlist - Grep words from WordList::*
 
 =head1 VERSION
 
-This document describes version 0.21 of App::wordlist (from Perl distribution App-wordlist), released on 2016-02-29.
+This document describes version 0.22 of App::wordlist (from Perl distribution App-wordlist), released on 2016-04-15.
 
 =head1 SYNOPSIS
 
@@ -315,7 +403,15 @@ Examples:
 
 =item * Select a specific wordlist (multiple -w allowed):
 
- wordlist( arg => ["foo"], wordlist => ["ID::KBBI"]);
+ wordlist( arg => ["foo"], wordlists => ["ID::KBBI"]);
+
+=item * Select phrase wordlists:
+
+ wordlist( arg => ["foo"], type => "Phrase");
+
+=item * Select French wordlists:
+
+ wordlist( arg => ["foo"], lang => "FR");
 
 =item * Filter by regex:
 
@@ -331,7 +427,7 @@ Examples:
 
 =back
 
-This function is not exportable.
+This function is not exported.
 
 Arguments ('*' denotes required arguments):
 
@@ -347,9 +443,13 @@ Display more information when listing modules.
 
 =item * B<ignore_case> => I<bool> (default: 1)
 
+=item * B<lang> => I<str>
+
+Only include wordlists of certain language.
+
 =item * B<lcpan> => I<bool>
 
-Use local CPAN mirror first when available.
+Use local CPAN mirror first when available (for -L).
 
 =item * B<len> => I<int>
 
@@ -361,7 +461,15 @@ Use local CPAN mirror first when available.
 
 Use OR logic instead of the default AND.
 
-=item * B<wordlist> => I<array[str]>
+=item * B<type> => I<str>
+
+Only include wordlists of certain type.
+
+By convention, language information is encoded in the wordlist's name. For
+example, English wordlists have names matching C<EN::*> or C<Word::EN::*> or
+C<Char::EN::*> or C<Phrase::EN::*>.
+
+=item * B<wordlists> => I<array[str]>
 
 Select one or more wordlist modules.
 
@@ -378,6 +486,10 @@ that contains extra information.
 
 Return value:  (any)
 
+=head1 ENVIRONMENT
+
+=head2 DEBUG => bool
+
 =head1 HOMEPAGE
 
 Please visit the project's homepage at L<https://metacpan.org/release/App-wordlist>.
